--                        -*- Mode: lua; lua-indent-level: 2; tab-width: 2 -*-
--[[
Combuctor_LibFilter

Author: Daphne Pfister <tiiroib02@sneakemail.com>

Adds LibFilter support to Combuctor

License

Copyright (c) 2008 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

-- Need to decide which SVN server revision number I will use and fix this
-- probably will got with svn.wowace.com's version for the long run
local MAJOR, MINOR = "Combuctor_LibFilter-1.0", tonumber(("$Revision: 25 $"):match("(%d+)"))
local lib = LibStub:NewLibrary(MAJOR,MINOR)
if not lib then return end

local LibFilter = LibStub("LibFilter-1.0")
local CombuctorSet

lib.known_rule_types = lib.known_rule_types or {}
lib.known_filters = lib.known_filters or {}
lib.known_presets = lib.known_presets or {}
lib.events = lib.events or {}

local known_rule_types = lib.known_rule_types
local known_filters = lib.known_filters
local known_presets = lib.known_presets
local events = lib.events
local do_upgrade = nil

local ITEM_TYPE = LibFilter.FilterType.ITEM

local function Print(msg)
	local info = ChatTypeInfo["SYSTEM"]
	DEFAULT_CHAT_FRAME:AddMessage(msg, info.r, info.g, info.b, info.id)
end
			
local function PresetRoot(presets,preset)
	if not preset then return nil end
	local parent = preset.parent
	if not parent then return preset.name end
	preset.parent = nil
	parent = PresetRoot(presets,presets[parent])
	preset.parent = parent
	return parent or preset.name
end

function lib:TestFilter(filter,type,preset,link)
	if type == ITEM_TYPE then
		return filter:TestIfmatch(preset,link) and true or false
	end
	return false
end

local function UpdatePreset(filter,type,presets,preset)
	if not filter then return end
	if not preset then return end
	if preset.updated then return end
	preset.updated = true
	if not preset.seen then
		if preset.set then
			CombuctorSet:Unregister(preset.old_desc,preset.old_parent_desc)
			preset.set = nil
		end
	end
	local parent = PresetRoot(presets,preset)
	local name = preset.name
	local desc = preset.desc
	if parent == name then parent = nil end
	local parent_preset = parent and presets[parent] or nil
	if parent_preset then
		if not parent_preset.seen then return end
		UpdatePreset(filter,type,presets,parent_preset)
	end
	local parent_desc = parent_preset and parent_preset.desc or nil
	if (desc ~= preset.old_desc) or (parent_desc ~= preset.old_parent_desc) then
		CombuctorSet:Unregister(preset.old_desc,preset.old_parent_desc)
		preset.set = nil
	end
	local icon = preset.icon  or "Interface\\Icons\\INV_Misc_QuestionMark"
	local func = function (_,_,_,link) return lib:TestFilter(filter,type,name,link) end
	if preset.set then
		msg = "Updating"
	else
		msg = "Registering"
	end
	if parent then
		--Print(("Combuctor_LibFilter: %s |cFFFFFFFF%s:%s|r"):format(msg,desc,parent_desc))
		if not CombuctorSet:Get(parent_desc) then
			--Print(("Combuctor_LibFilter: Could not %s |cFFFFFFFF%s:%s|r"):format(msg,desc,parent_desc))
		else
			CombuctorSet:RegisterSubSet(desc,parent_desc,icon,func)
			preset.set = CombuctorSet:Get(desc,parent_desc)
		end
	else
		--Print(("Combuctor_LibFilter: %s |cFFFFFFFF%s|r"):format(msg,desc))
		CombuctorSet:Register(desc,icon,func)
		preset.set = CombuctorSet:Get(desc)
	end
end

function lib:UpdatePresets(filtername,filter,type)
	known_presets[filtername] = known_presets[filtername] or {}
	local presets = known_presets[filtername]

	-- Mark all existing presets as unseen incase the need to be removed
	for _,preset in pairs(presets) do
		preset.seen = nil
		preset.updated = nil
		preset.old_desc = preset.desc
		preset.old_parent = preset.parent
		preset.old_parent_desc = preset.parent_desc
	end

	-- Populate presets
	local maxdepth = 0
	for name,parent in filter:IteratePresets(type) do
		maxdepth = maxdepth + 1
		local desc,icon
		if filter.DescribeSettings then
			desc,icon = filter:DescribeSettings(name)
		end
		desc = desc or preset
		if desc == "" then
			desc = filter.name or filter.key
		end
		--Print(("Combuctor_LibFilter: Saw |cFFFFFFFF%s:%s|r"):format(desc,parent or "root"))
		presets[name] = presets[name] or {}
		local preset = presets[name]
		preset.seen = true
		preset.desc = desc
		preset.name = name
		preset.icon = icon
		if parent and parent ~= presetname then
			-- Nested preset
			preset.parent = parent
		else
			-- Base preset
			preset.parent = nil
		end
	end

	-- Update sets
	for _,preset in pairs(presets) do
		UpdatePreset(filter,type,presets,preset)
	end

	-- Remove old presets
	for name,preset in pairs(presets) do
		if not preset.seen then
			presets[name] = nil
		end
	end
end

local function AddFilterRuleType(filtername,filter)
	-- Check to see if filter support presets
	if not filter.IteratePresets then return end

	if known_filters[filtername] and not do_upgrade then return end
	known_filters[filtername] = filter

	prefered_type = ITEM_TYPE

	filter.RegisterCallback(lib,"PresetUpdate",function() lib:UpdatePresets(filtername,filter,prefered_type) end)
	filter.RegisterCallback(lib,"FilterUpdate",function() lib:UpdatePresets(filtername,filter,prefered_type) end)
	lib:UpdatePresets(filtername,filter,prefered_type)

	Print(("Combuctor_LibFilter: Registered |cFFFFFFFF%s|r"):format(filtername))
end

local function UpdateFilters()
	if not CombuctorSet then
		if Combuctor then
			CombuctorSet = Combuctor:GetModule('Sets')
		end
	end
	if CombuctorSet then
		for filtername,filter in LibFilter:IterateFilters(ITEM_TYPE) do
			AddFilterRuleType(filtername,filter)
		end
		do_upgrade = nil
	end
end

lib.myFrame = lib.myFrame or CreateFrame("Frame","Combuctor_LibFilter_Frame")
local myFrame = lib.myFrame
myFrame:SetScript("OnEvent",function (self, event, ...) events[event](...) end)

function events.UPDATE()
	UpdateFilters()
	myFrame:SetScript("OnUpdate",nil)
end

local function update_next_frame()
	myFrame:SetScript("OnUpdate",function (self, ...) events.UPDATE(...) end)
end

function events.ADDON_LOADED()
	update_next_frame()
end

function events.PLAYER_LOGIN()
	update_next_frame()
	myFrame:RegisterEvent("ADDON_LOADED")
end

do_upgrade = true
if IsLoggedIn() then
	update_next_frame()
	myFrame:RegisterEvent("ADDON_LOADED")
else
	myFrame:RegisterEvent("PLAYER_LOGIN")
end
